@@ -7,10 +7,17 @@
     my $what = shift;
     $what eq 'tests' or die "cannot plan anything but tests";
     $__total = shift;
+    if (defined $__total && $__total eq 'no_plan') {
+      return;
+    }
     defined $__total && $__total > 0 or die "need a positive number of tests";
     print "1..$__total\n";
   }
 
+  sub done_testing {
+    print "1..",$__ntest,"\n";
+  }
+
   sub skip {
     my $reason = shift;
     ++$__ntest;
@@ -21,6 +28,7 @@
     local($\,$,);
     my $ok = 0;
     my $result = shift;
+    my $comment = '';
     if (@_ == 0) {
       $ok = $result;
     } else {
@@ -30,17 +38,19 @@
       } elsif (!defined $result) {
         $ok = 0;
       } elsif (ref($expected) eq 'Regexp') {
-        die "using regular expression objects is not backwards compatible";
+        $ok = $result =~ $expected;
+        #die "using regular expression objects is not backwards compatible";
       } else {
         $ok = $result eq $expected;
       }
+      $comment = ' # '.shift if @_;
     }
     ++$__ntest;
     if ($ok) {
-      print "ok $__ntest\n"
+      print "ok $__ntest$comment\n"
     }
     else {
-      print "not ok $__ntest\n"
+      print "not ok $__ntest$comment\n"
     }
     $ok
   }
